/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot;

import java.rmi.RemoteException;
import java.text.DateFormat;
import java.text.DecimalFormat;

import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.I18N;

/**
 * Settings Klasse fr den jameica.qrscanner Basiert auf den Settings-Klasse aus
 * dem Beispiel-Plugin von Olaf Willuhn.
 * 
 * @author willuhn/munstfr
 * @version 1.0
 * @see <a href=
 *      "http://www.willuhn.de/products/jameica/dev.php">http://www.willuhn.de/products/jameica/dev.php</a>
 * 
 * 
 */
public final class Settings {

	/**
	 * Our currency name.
	 */
	public final static String CURRENCY = "EUR"; //$NON-NLS-1$

	/**
	 * Our DateFormatter.
	 */
	public final static DateFormat DATEFORMAT = DateFormat.getDateInstance(DateFormat.DEFAULT,
			Application.getConfig().getLocale());

	/**
	 * Our decimal formatter.
	 */
	public final static DecimalFormat DECIMALFORMAT = (DecimalFormat) DecimalFormat
			.getInstance(Application.getConfig().getLocale());

	/**
	 * Unser Datenbank-Service
	 */
	private static DBService dbService;

	/***
	 * Ist Hibiscus auch installiert?
	 */
	private static Boolean hibiscusInstalled = Application.getPluginLoader()
			.isInstalled("de.willuhn.jameica.hbci.HBCI"); //$NON-NLS-1$

	/**
	 * Der bersetzer
	 */
	private static I18N i18n = Application.getPluginLoader().getPlugin(JaDepot.class).getResources().getI18N();

	/**
	 * Settings Objekt von Jameica
	 */
	private static de.willuhn.jameica.system.Settings settings = Application.getPluginLoader().getPlugin(JaDepot.class) //NOPMD
			.getResources().getSettings();

	/**
	 * Small helper function to get the database service.
	 * 
	 * @return db service.
	 * @throws RemoteException
	 */
	public static DBService getDBService() throws RemoteException {

		if (dbService != null) {
			return dbService; // NOPMD by munstfr on 14.09.16 14:38
		}

		try {
			// We have to ask Jameica's ServiceFactory.
			// If we are running in Client/Server mode and we are the
			// client, the factory returns the remote dbService from the
			// Jameica server.
			// The name and class of the service is defined in plugin.xml
			dbService = (DBService) Application.getServiceFactory().lookup(JaDepot.class, "jaDepotDB"); //$NON-NLS-1$
			return dbService;
		} catch (Exception e) { // NOPMD
			throw new RemoteException(Settings.getI18n().tr("Settings.errorDbService"), e); //$NON-NLS-1$
		}
	}

	/**
	 * Liefert die Standardw&auml;hrung.
	 * 
	 * @return Standard-W&auml;hrung, default ist <code>EUR</code>.
	 */
	public static String getDepotWaehrung() {
		return CURRENCY;
	}

	/**
	 * Small helper function to get the translator.
	 * 
	 * @return translator.
	 */
	public static I18N getI18n() {
		return i18n;
	}

	/**
	 * berprft, ob das Hibiscus -Plugin installiert ist.
	 * 
	 * @return isHibiscusInstalled <b>true</b>, wenn Hibiscus installiert ist,
	 *         <b>false</b> sonst
	 */
	public static Boolean isHibiscusInstalled() {
		return hibiscusInstalled;
	}

	/**
	 * Liefert die showHibiscusSummery.
	 * 
	 * @return showHibiscusSummery, ist immer <code>false</code>, falls Hibiscus
	 *         gar nicht installiert ist.
	 */
	public static boolean isShowHibiscusSummery() {
		boolean isShowHibiscus = false;
		if (isHibiscusInstalled()) {
			isShowHibiscus = settings.getBoolean("JaDepot.showHibiscusSummery", true); //$NON-NLS-1$
		}
		return isShowHibiscus;

	}

	/**
	 * Setzt showHibiscusSummery.
	 * 
	 * @param showHibiscus
	 *            soll Hibsicus Kontenzusammenfassung angezeigt werden.
	 */
	public static void setShowHibiscusSummery(final Boolean showHibiscus) {
		settings.setAttribute("JaDepot.showHibiscusSummery", //$NON-NLS-1$
				showHibiscus);
	}

	private Settings() {
	}
}
