/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Version;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Implementierung eines Versionsdatensatzes.
 * 
 * @author Frank Mnster
 * @version 1.0
 */
public class VersionImpl extends AbstractDBObject implements Version

{

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = -2807984014766080725L;

	/**
	 * Unser bersetzer
	 */
	private final static I18N I18N = Settings.getI18n();

	/**
	 * ct
	 * 
	 * @throws RemoteException
	 */
	public VersionImpl() throws RemoteException {
		super();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return "name"; //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
	 */
	@Override
	protected String getTableName() {
		return "version"; //$NON-NLS-1$
	}

	/**
	 * @see Version#getName()
	 */
	@Override
	public String getName() throws RemoteException {
		return (String) getAttribute("name"); //$NON-NLS-1$
	}

	/**
	 * @see Version#getVersion()
	 */
	@Override
	public int getVersion() throws RemoteException {
		// Wir fangen bei 0 an zu zaehlen
		Integer version = null;
		try {
			version = (Integer) getAttribute("version"); //$NON-NLS-1$
		} catch (RemoteException e) {
			Logger.warn(Settings.getI18n().tr("VersionImpl.noDbVersionFound")); //$NON-NLS-1$
		}
		return version == null ? 0 : version.intValue();
	}

	/**
	 * @see Version#setVersion(int)
	 */
	@Override
	public void setVersion(final int newVersion) throws RemoteException {
		if (newVersion < 0) {
			throw new RemoteException(Settings.getI18n().tr(
					"VersionImpl.versionSmallerZero")); //$NON-NLS-1$
		}
		setAttribute("version", Integer.valueOf(newVersion)); //$NON-NLS-1$
	}

	/**
	 * @see Version#setName(java.lang.String)
	 */
	@Override
	public void setName(final String name) throws RemoteException {
		setAttribute("name", name); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
	 */
	@Override
	protected void insertCheck() throws ApplicationException {

		try {
			if (getName() == null || getName().length() == 0) {
				throw new ApplicationException(I18N.tr(Settings.getI18n().tr(
						"VersionImpl.noVersionGiven"))); //$NON-NLS-1$
			}
		} catch (RemoteException re) {
			Logger.error(
					Settings.getI18n().tr("VersionImpl.errorCheckVersion"), re); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(
					I18N.tr(Settings.getI18n().tr(
							"VersionImpl.errorCheckVersion"))).initCause(re); //$NON-NLS-1$
		}
		super.insertCheck();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
	 */
	@Override
	protected void updateCheck() throws ApplicationException {
		insertCheck();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insert()
	 */
	@Override
	public void insert() throws RemoteException, ApplicationException {
		setVersion(getVersion()); // speichert automatisch die Startnummer
		super.insert();
	}

}

/*********************************************************************
 * $Log: VersionImpl.java,v $ Revision 1.2 2007/12/11 00:33:35 willuhn
 * 
 * @N Scharfschaltung des neuen Update-Prozesses
 * 
 *    Revision 1.1 2007/12/06 17:57:21 willuhn
 * @N Erster Code fuer das neue Versionierungs-System
 * 
 **********************************************************************/
