/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Version;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.util.ApplicationException;

/**
 * Utility class fr die Datenbank Versionierung.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public final class VersionUtil {

	private VersionUtil() {
		// Utility class
	}

	/**
	 * Liefert die Version der Komponenten. Ist diese noch nicht in der DB, wird
	 * sie mit Version 1 angelegt.
	 * 
	 * @param service
	 * 
	 * @param name
	 *            Name der Komponenten deren Version geliefert wwerden soll.
	 * @return die Version
	 * @throws RemoteException
	 * @throws ApplicationException
	 */
	public static Version getVersion(final DBService service, final String name)
			throws RemoteException, ApplicationException {
		if (name == null || name.length() == 0) {
			throw new ApplicationException(Settings.getI18n().tr(
					Settings.getI18n().tr("VersionUtil.versionNullOrEmpty"))); //$NON-NLS-1$
		}
		Version version;
		try {
			final DBIterator<?> list = service.createList(Version.class);
			list.addFilter("name = ?", name); //$NON-NLS-1$
			if (list.hasNext()) {
				version = (Version) list.next();
			} else {

				// Neue Version erstellen
				version = (Version) service.createObject(Version.class, null);
				version.setName(name);
				version.store();
			}
		} catch (RemoteException e) {
			// Version 0 erstellen, da Versionstabelle noch nicht existiert.
			version = new VersionImpl();
			version.setName("db"); //$NON-NLS-1$
			version.setVersion(0);
		}
		return version;
	}
}
