/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.finanzen.currencyexchangerate;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

/**
 * Enum fuer die verfuegbaren W&auml;hrungen.
 */
public enum Currency {
	/**
	 * Euro
	 */
	EUR("EUR", CurrencyName.getString("Currency.eur")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * US Dolloar
	 */
	USD("USD", CurrencyName.getString("Currency.usd")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Lew (Bulgarien)
	 */
	BGN("BGN", CurrencyName.getString("Currency.bgn")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Tschechische Krone
	 */
	CZK("CZK", CurrencyName.getString("Currency.czk")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Dnische Krone
	 */
	DKK("DKK", CurrencyName.getString("Currency.dkk")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Pfund Sterling
	 */
	GBP("GBP", CurrencyName.getString("Currency.gbp")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Forint (Ungarn)
	 */
	HUF("HUF", CurrencyName.getString("Currency.huf")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Zloty (Polen
	 */
	PLN("PLN", CurrencyName.getString("Currency.pln")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Rumnischer Leu
	 */
	RON("RON", CurrencyName.getString("Currency.ron")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Schwedische Krone
	 */
	SEK("SEK", CurrencyName.getString("Currency.sek")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Schweizer Franken
	 */
	CHF("CHF", CurrencyName.getString("Currency.chf")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Norwegische Krone
	 */
	NOK("NOK", CurrencyName.getString("Currency.nok")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Kuna (Kroatien)
	 */
	HRK("HRK", CurrencyName.getString("Currency.hrk")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Russischer Rubel
	 */
	RUB("RUB", CurrencyName.getString("Currency.rub")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Neue Trkische Lira
	 */
	TRY("TRY", CurrencyName.getString("Currency.rty")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Australischer Dollar
	 */
	AUD("AUD", CurrencyName.getString("Currency.aud")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Brasilianischer Real
	 */
	BRL("BRL", CurrencyName.getString("Currency.brl")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Kanadischer Dollar
	 */
	CAD("CAD", CurrencyName.getString("Currency.cad")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Chinesischer Yuan
	 */
	CNY("CNY", CurrencyName.getString("Currency.cny")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Hongkong-Dollar
	 */
	HKD("HKD", CurrencyName.getString("Currency.hkd")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Indonesische Rupiah
	 */
	IDR("IDR", CurrencyName.getString("Currency.idr")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Neuer Israel Scheke
	 */
	ILS("ILS", CurrencyName.getString("Currency.ils")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Indische Rupie
	 */
	INR("INR", CurrencyName.getString("Currency.inr")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Sdkoreanischer Won
	 */
	KRW("KRW", CurrencyName.getString("Currency.krw")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Mexikanischer Peso
	 */
	MXN("MXN", CurrencyName.getString("Currency.mxn")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Malaysische Ringgit
	 */
	MYR("MYR", CurrencyName.getString("Currency.myr")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Neuseeland-Dollar
	 */
	NZD("NZD", CurrencyName.getString("Currency.nzd")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Philippinischer Peso
	 */
	PHP("PHP", CurrencyName.getString("Currency.php")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Singapur-Dollar
	 */
	SGD("SGD", CurrencyName.getString("Currency.sgd")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Thailndische Baht
	 */
	THB("THB", CurrencyName.getString("Currency.thb")), //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * Sdafrikanischer RAND
	 */
	ZAR("ZAR", CurrencyName.getString("Currency.zar")); //$NON-NLS-1$ //$NON-NLS-2$

	// private final static I18N I18N = Settings.getI18n();

	/**
	 * Der W&auml;hrungscode nach ISO-4217
	 */
	private String currency;

	/**
	 * Der Name der W&auml;hrung (freie Nameswahl)
	 */
	private String name;

	/**
	 * ct.
	 * 
	 * @param currency
	 *            die W&auml;hrung.
	 * @param name
	 *            die sprechende Bezeichnung.
	 */
	private Currency(final String currency, final String name) {
		this.currency = currency;
		this.name = name;
	}

	/**
	 * Liefert die W&auml;hrung.
	 * 
	 * @return currency ddie W&auml;hrung.
	 */
	public String getCode() {
		return currency;
	}

	/**
	 * Liefert einen sprechenden Namen f&uuml;r die W&auml;hrung.
	 * 
	 * @return sprechender Name f&uuml;r die W&auml;hrung.
	 */
	public String getName() {
		// return I18N.tr(this.name);
		return this.name;
	}

	/**
	 * Ermittelt den die W&auml;hrungs-Code.
	 * 
	 * @param currencyName
	 *            der die W&auml;hrung. Kann NULL sein.
	 * @return der W&auml;hrungs-Code oder NULL, wenn der Code nicht bekannt
	 *         ist.
	 */
	public static Currency find(final String currencyName) {

		Currency currency = null;
		if (StringUtils.trimToNull(currencyName) != null) {
			for (final Currency pc : values()) {
				if (pc.getCode().equals(currencyName)) {
					currency = pc;
					break;
				}
			}
		}
		return currency;
	}

	/**
	 * @see java.lang.Enum#toString()
	 */
	@Override
	public String toString() {
		return this.getName();
	}

	/**
	 * Liefert die Liste der bekannten Codes.
	 * 
	 * @return die Liste der bekannten Codes.
	 */
	public static List<String> codes() {
		final List<String> result = new ArrayList<String>();
		for (final Currency c : values()) {
			result.add(c.getCode());
		}

		return result;
	}

}
