/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.finanzen.yahoo.finance;

import java.util.Date;

/**
 * Ein Wertpapier-Wert Bean
 * 
 * @author munstfr
 * @version 1.0
 */
public class HistoryValueBean implements Comparable<HistoryValueBean> {

	/**
	 * Das Yahoo-Symbol
	 */
	private String symbol;

	/**
	 * Das Datum
	 */
	private Date date;

	/**
	 * Der Er&ouml;ffnugskurs
	 */
	private double open;

	/**
	 * Der Tagesh&ouml;chststand
	 */
	private double high;

	/**
	 * Der Tagesniedrigsstand
	 */
	private double low;

	/**
	 * Der Schlusskurs
	 */
	private double close;

	/**
	 * Das Umsatzvolumen
	 */
	private double volume;

	/**
	 * der angepasste Schusskurs
	 */
	private double adjClose;

	/**
	 * Standartkonstruktor
	 */
	public HistoryValueBean() {
		// absichtlich leer;
	}

	/**
	 * Liefert das Yahoo-Symbol
	 * 
	 * @return symbol das Yahoo-Symbol
	 */
	public String getSymbol() {
		return symbol;
	}

	/**
	 * Setzt das Yahoo-Symbol
	 * 
	 * @param symbol
	 *            das Yahoo-Symbol
	 */
	public void setSymbol(final String symbol) {
		this.symbol = symbol;
	}

	/**
	 * Liefert das Datum
	 * 
	 * @return date das datum
	 */
	public Date getDate() {
		return date;
	}

	/**
	 * Setzt das Datum
	 * 
	 * @param date
	 *            das Datum
	 */
	public void setDate(final Date date) {
		this.date = date;
	}

	/**
	 * Liefert den Er&ouml;ffnugskurs
	 * 
	 * @return open Der Er&ouml;ffnugskurs
	 */
	public double getOpen() {
		return open;
	}

	/**
	 * Setzt den Er&ouml;ffnugskurs
	 * 
	 * @param open
	 *            Der Er&ouml;ffnugskurs
	 */
	public void setOpen(final double open) {
		this.open = open;
	}

	/**
	 * Liefert den Tagesh&ouml;chststand
	 * 
	 * @return high der Tagesh&ouml;chststand
	 */
	public double getHigh() {
		return high;
	}

	/**
	 * Setzt den Tagesh&ouml;chststand
	 * 
	 * @param high
	 *            der Tagesh&ouml;chststand
	 */
	public void setHigh(final double high) {
		this.high = high;
	}

	/**
	 * Liefert den Tagesniedrigsstand
	 * 
	 * @return low der Tagesniedrigsstand
	 */
	public double getLow() {
		return low;
	}

	/**
	 * Setzte den Tagesniedrigsstand
	 * 
	 * @param low
	 *            der Tagesniedrigsstand
	 */
	public void setLow(final double low) {
		this.low = low;
	}

	/**
	 * Liefert den Schlusskurs
	 * 
	 * @return close der Schlusskurs
	 */
	public double getClose() {
		return close;
	}

	/**
	 * Setzt den Schlusskurs
	 * 
	 * @param close
	 *            der Schlusskurs
	 */
	public void setClose(final double close) {
		this.close = close;
	}

	/**
	 * Liefert das Umsatzvolumen
	 * 
	 * @return volume das Umsatzvolumen
	 */
	public double getVolume() {
		return volume;
	}

	/**
	 * Setzt das Umsatzvolumen
	 * 
	 * @param volume
	 *            das Umsatzvolumen
	 */
	public void setVolume(final double volume) {
		this.volume = volume;
	}

	/**
	 * Liefert den angepasste Schlusskurs
	 * 
	 * @return adjClose der angepasste Schlusskurs
	 */
	public double getAdjClose() {
		return adjClose;
	}

	/**
	 * Setzt den angepasste Schlusskurs
	 * 
	 * @param adjClose
	 *            der angepasste Schlusskurs
	 */
	public void setAdjClose(final double adjClose) {
		this.adjClose = adjClose;
	}

	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(final HistoryValueBean historyValueBean) {

		int compareTo;

		if (historyValueBean.getDate() == null && this.getDate() == null) {
			compareTo = 0;
		} else if (this.getDate() == null) {
			compareTo = 1;
		} else if (historyValueBean.getDate() == null) {
			compareTo = -1;
		} else {
			compareTo = this.getDate().compareTo(historyValueBean.getDate());
		}

		return compareTo;
	}
}
