/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.finanzen.yahoo.finance;

/**
 * Yahoo Wertpapier Bean
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class StockInfoBean {

	/**
	 * B&ouml;senplatz
	 */
	private String exchange;

	/**
	 * Wertpapiername
	 */
	private String name;

	/**
	 * Yahho Symbole
	 */
	private String symbol;

	/**
	 * Wertpapiertyp
	 */
	private String type;

	/**
	 * Standartkonstruktor
	 */
	public StockInfoBean() {
		this(null, null, null, null);
	}

	/**
	 * Konstruktor mit Angabe aller Felder.
	 * 
	 * @param symbol Das Yahoo-Symbol
	 * @param name der Wertpapiername
	 * @param exchange Der B&ouml;rsenplatz
	 * @param type Der Wertpapiertyp
	 */
	public StockInfoBean(final String symbol, final String name, final String exchange, final String type) {
		this.symbol = symbol;
		this.name = name;
		this.exchange = exchange;
		this.type = type;
	}

	/**
	 * Liefert den b&ouml;rsenplatz 
	 * @return der b&ouml;rsenplatz
	 */
	public String getExchange() {
		return exchange;
	}

	/**
	 * Liefert den Wertpapiernamen
	 * @return der Wertpapiernamen
	 */
	public String getName() {
		return name;
	}

	/**
	 * Liefert das Yahoo-Symbol
	 * @return das Yahoo-Symbol
	 */
	public String getSymbol() {
		return symbol;
	}

	/**
	 * Liefert den Wertpapiertyp
	 * @return der Wertpapiertyp
	 */
	public String getType() {
		return type;
	}

	/**
	 * Setzt den b&ouml;rsenplatz
	 * @param exchange der b&ouml;rsenplatz
	 */
	public void setExchange(final String exchange) {
		this.exchange = exchange;
	}

	/**
	 * Setzt den Wertpapiernamen
	 * @param name der Wertpapiernamen
	 */
	public void setName(final String name) {
		this.name = name;
	}

	/**
	 * Setzt das Yahoo-Symbol 
	 * @param symbol das Yahoo-Symbol
	 */
	public void setSymbol(final String symbol) {
		this.symbol = symbol;
	}

	/**
	 * Setzt den Wertpapiertyp
	 * @param type der Wertpapiertyp
	 */
	public void setType(final String type) {
		this.type = type;
	}

	/**
	 * @see Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer stringBuffer = new StringBuffer();
		if (symbol != null) {
			stringBuffer.append(symbol);
		}
		if (name != null) {
			stringBuffer.append('\t');
			stringBuffer.append(name);
		}
		if (exchange != null) {
			stringBuffer.append('\t');
			stringBuffer.append(exchange);
		}
		if (type != null) {
			stringBuffer.append('\t');
			stringBuffer.append(type);
		}
		return stringBuffer.toString();
	}

}
