/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.action;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.gui.action.listener.OrderDeleteListener;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Aktion zum L&ouml;schen einer Order.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class OrderDeleteAction implements Action {

	/**
	 * Die Listener fr einen QrCodeFound-Event
	 */
	private final transient Set<OrderDeleteListener> listeners = new CopyOnWriteArraySet<OrderDeleteListener>();

	/**
	 * Konstruktor mit Listener
	 * 
	 * @param listener
	 *            der OrderDeleteListener
	 */
	public OrderDeleteAction(final OrderDeleteListener listener) {
		this.addListener(listener);
	}

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public final void addListener(final OrderDeleteListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public final void removeListener(final OrderDeleteListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrieretn Listeners mit bergabe des gefundenen
	 * Strings
	 */
	private final void notifyListeners() {
		for (final OrderDeleteListener listener : listeners) {
			listener.notifyOrderDeleted();
		}
	}

	/**
	 * 
	 */
	@Override
	public void handleAction(final Object context) throws ApplicationException {

		// check if the context is a project
		if (!(context instanceof OrderHistory)) {
			throw new ApplicationException(Settings.getI18n().tr(
					"OrderDeleteAction.chooseOrder")); //$NON-NLS-1$
		}

		final OrderHistory order = (OrderHistory) context;

		try {

			// before deleting the order, we show up a confirm dialog ;)
			final String question = Settings.getI18n().tr(
					"OrderDeleteAction.reallyDelete"); //$NON-NLS-1$
			if (!Application.getCallback().askUser(question)) {
				return;
			}

			order.delete();

			// Send Status update message
			Application.getMessagingFactory().sendMessage(
					new StatusBarMessage(Settings.getI18n().tr(
							"OrderDeleteAction.deleteSuccessful"), //$NON-NLS-1$
							StatusBarMessage.TYPE_SUCCESS));
		} catch (Exception e) { // NOPMD
			Logger.error(
					Settings.getI18n().tr("OrderDeleteAction.deleteError"), e); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(Settings
					.getI18n().tr("OrderDeleteAction.deleteError")).initCause(e); //$NON-NLS-1$
		}

		notifyListeners();
	}

}
