/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.controller;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Locale;

import org.eclipse.swt.SWT;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.action.WertpapierDetailAction;
import de.frankmuenster.jameica.depot.gui.action.listener.WertpapierAktualisiertListener;
import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.gui.parts.WertpapierListePart;
import de.frankmuenster.jameica.depot.gui.parts.column.ExchangeInfo;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.logging.Logger;

/**
 * Kontroller f&uumr; die Hauptseite
 * 
 * @author munstfr
 * @version 1.0
 */
public class DepotController extends AbstractControl implements
		WertpapierAktualisiertListener {

	/**
	 * Wechselkurs Info
	 */
	private transient TextAreaInput wechselKursInfo;

	/**
	 * Hibiscus Saldo
	 */
	private transient LabelInputExtended hibiscusTotal;

	/**
	 * JaDepot Saldo
	 */
	private transient LabelInputExtended jaDepotTotal;

	/**
	 * Hibiscus + JaDepot Saldo
	 */
	private transient LabelInputExtended hibisJaDepotTotal;

	/**
	 * Liste der Wertpapiere
	 */
	private transient WertpapierListePart wpListePart;

	/**
	 * Konstruktor mit View
	 * 
	 * @param view
	 */
	public DepotController(final AbstractView view) {
		super(view);
	}

	/**
	 * Liefert die Tabelle mit den Wertpapieren.
	 * 
	 * @return the wpListePart
	 */
	public WertpapierListePart getAktienListe() {
		if (wpListePart == null) {
			try {
				wpListePart = new WertpapierListePart(
						new WertpapierDetailAction());
			} catch (RemoteException e) {
				Logger.error(e.getMessage());
			}
		}
		return wpListePart;
	}

	/**
	 * Liefert die Wechselkurs Info anzeige.
	 * 
	 * @return TextAreaInput wechselKursInfo
	 */
	public TextAreaInput getWechselKursInfos() {
		if (wechselKursInfo == null) {

			final StringBuffer waehrungInfo = new StringBuffer();
			if (getAktienListe().isShowWaehrungsSpalte()) {
				final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
						Locale.GERMAN);
				final DecimalFormat dateFormat = new DecimalFormat(
						"###,###,##0.0000"); //$NON-NLS-1$
				final List<ExchangeInfo> exchangeInfos = getAktienListe()
						.getExchangeInfos();
				for (final ExchangeInfo exchangeInfo : exchangeInfos) {
					waehrungInfo
							.append(exchangeInfo.getWaehrungFrom())
							.append(' ')
							.append(Settings.getI18n().tr("DepotController.to")) //$NON-NLS-1$
							.append(' ').append(exchangeInfo.getWaehrungTo())
							.append(":   ") //$NON-NLS-1$
							.append(dateFormat.format(exchangeInfo.getKurs()));
					if (exchangeInfo.getDatum() != null) { // NOPMD
						waehrungInfo
								.append(", ") //$NON-NLS-1$
								.append(Settings.getI18n().tr(
										"DepotController.date")) //$NON-NLS-1$
								.append(": ") //$NON-NLS-1$
								.append(sdf.format(exchangeInfo.getDatum()));
					}
					waehrungInfo.append('\n');
				}
			}

			wechselKursInfo = new TextAreaInput(waehrungInfo.toString());
			wechselKursInfo.setComment(""); //$NON-NLS-1$
			wechselKursInfo.setEnabled(false);
			wechselKursInfo.setHeight(70);
		}
		return wechselKursInfo;

	}

	/**
	 * Liefert die Saldoanzeige f&uuml; die Hibiscus-Konten
	 * 
	 * @return LabelInputExtended hibiscusTotal
	 */
	public LabelInputExtended getHibiscusTotal() {
		if (this.hibiscusTotal == null) {

			
			BigDecimal hibiscusTotal = new BigDecimal("0.0"); //$NON-NLS-1$
			
			if (Settings.isHibiscusInstalled()) {
				try {
					final DBIterator<?> hibiscusKontos = de.willuhn.jameica.hbci.Settings
							.getDBService().createList(
									de.willuhn.jameica.hbci.rmi.Konto.class);
					// i1.setOrder("ORDER BY LOWER(kategorie), blz, kontonummer, bezeichnung");
					while (hibiscusKontos.hasNext()) {
						final de.willuhn.jameica.hbci.rmi.Konto konto = (de.willuhn.jameica.hbci.rmi.Konto) hibiscusKontos
								.next();
						Logger.warn(Settings.getI18n().tr(
								"DepotController.logAccountBalance", //$NON-NLS-1$
								konto.getName(),
								Double.toString(konto.getSaldo())));
						hibiscusTotal = hibiscusTotal.add(new BigDecimal(konto.getSaldo())); //NOPMD
					}
				} catch (RemoteException e) {
					Logger.error(Settings.getI18n().tr(
							"DepotController.getBalanceError" + e.getMessage())); //$NON-NLS-1$
					hibiscusTotal = new BigDecimal("0.0"); //$NON-NLS-1$
				}
			} else {
				hibiscusTotal = new BigDecimal("0.0"); //$NON-NLS-1$
			}

			this.hibiscusTotal = new LabelInputExtended(" " //$NON-NLS-1$
					+ FormatterUtils.DECIMAL_FORMAT.get().format(hibiscusTotal), true, SWT.RIGHT);
			this.hibiscusTotal.setComment(Settings.getDepotWaehrung()); 
			this.hibiscusTotal.setEnabled(false);

		}
		return this.hibiscusTotal;
	}

	/**
	 * Liefert das Sadlo der JaDepot-Wertpapiere
	 * 
	 * @return LabelInputExtended jaDepotTotal
	 */
	public LabelInputExtended getJaDepotTotal() {
		if (jaDepotTotal == null) {

			jaDepotTotal = new LabelInputExtended(" " //$NON-NLS-1$
					+ FormatterUtils.DECIMAL_FORMAT.get().format(getAktienListe()
							.getGesamtSumme()), true, SWT.RIGHT);
			jaDepotTotal.setComment(Settings.getDepotWaehrung()); 
			jaDepotTotal.setEnabled(false);
		}
		return jaDepotTotal;
	}

	/**
	 * Liefert die Summe der Salden.
	 * 
	 * @return LabelInputExtended hibisJaDepotTotal
	 */
	public LabelInputExtended getHibiscusJaDepotTotal() {
		if (hibisJaDepotTotal == null) {

			Double total = 0.0d;
			if (Settings.isHibiscusInstalled()) {
				String totalString = (String) this.hibiscusTotal.getValue();
				totalString = totalString.replace(".", ""); //$NON-NLS-1$ //$NON-NLS-2$
				totalString = totalString.replace(",", "."); //$NON-NLS-1$ //$NON-NLS-2$
				total = Double.parseDouble(totalString);

			}
			total += getAktienListe().getGesamtSumme();

			hibisJaDepotTotal = new LabelInputExtended(" " //$NON-NLS-1$
					+ FormatterUtils.DECIMAL_FORMAT.get().format(total), true, SWT.RIGHT);
			hibisJaDepotTotal.setComment(Settings.getDepotWaehrung());
			hibisJaDepotTotal.setEnabled(false);
		}
		return hibisJaDepotTotal;

	}

	/**
	 * @see WertpapierAktualisiertListener #notifyWertpapiereAktualisiert()
	 */
	@Override
	public void notifyWertpapiereAktualisiert() {
		// die Tabelle neu laden
		final WertpapierListePart wpListe = getAktienListe();

		try {

			wpListe.removeAll();

			JaDepotDBService service;
			service = (JaDepotDBService) Settings.getDBService();
			final DBIterator<?> list = service.createList(Wertpapier.class);
			list.setOrder("ORDER BY symbol desc"); //$NON-NLS-1$

			while (list.hasNext()) {
				final Wertpapier wertpapier = (Wertpapier) list.next();
				wpListe.addItem(wertpapier);
			}
		} catch (RemoteException e) {
			Logger.error(e.getMessage());
		}
	}

}
