/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.Settings;
import de.willuhn.io.FileFinder;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.util.InfoReader;
import de.willuhn.logging.Logger;

/**
 * Controller fuer den Lizenzinformationen Dialog
 * 
 * Basiert auf dem
 * {@link de.willuhn.jameica.gui.internal.controller#LincenseControl}
 * 
 * @author willuhn/munstfr
 * @version 1.0
 * 
 */
public class LicenseControl extends AbstractControl {

	/**
	 * HMTL Line break
	 */
	private final static String HTML_BR = "<br />"; //$NON-NLS-1$

	/**
	 * FormTextPart um dem Text schn formatieren zu knnen
	 */
	private transient FormTextPart libList;

	/**
	 * Standrdart Konstruktor liefert ein LincenceControl object
	 * 
	 * @param view
	 *            die View, fuer die dieser LicenceControl zustaendig ist
	 */
	public LicenseControl(final AbstractView view) {
		super(view);
	}

	/**
	 * Liefert eine Liste mit allen direkt von jameica.qrscanner verwendeten
	 * Komponenten.
	 * 
	 * @return Liste der Komponenten.
	 */
	public Part getLibList() {
		if (libList == null) {

			final AbstractPlugin plugin = Application.getPluginLoader()
					.getPlugin(JaDepot.class);

			final StringBuffer buffer = new StringBuffer(
					"<form><p><span color=\"header\" font=\"header\">JaDepot</span></p>"); //$NON-NLS-1$

			Manifest manifest = null;
			manifest = plugin.getManifest();
			if (manifest != null) {
				buffer.append("<p>"); //$NON-NLS-1$
				buffer.append(manifest.getDescription());
				buffer.append(HTML_BR).append(manifest.getHomepage());
				buffer.append(HTML_BR).append(manifest.getLicense());
				buffer.append("</p>"); //$NON-NLS-1$
			}

			final String path = plugin.getManifest().getPluginDir();

			final FileFinder finder = new FileFinder(new File(path + "/lib")); //$NON-NLS-1$
			finder.matches(".*?info\\.xml$"); //$NON-NLS-1$
			final File[] infos = finder.findRecursive();
			for (int i = 0; i < infos.length; ++i) {
				if (!infos[i].isFile() || !infos[i].canRead()) {
					Logger.warn(Settings
							.getI18n()
							.tr("LicenseControl.infoFileReadError", infos[i].getName())); //$NON-NLS-1$
					continue;
				}

				try {
					final InputStream infoXml = new FileInputStream( // NOPMD
							infos[i]);
					final InfoReader infoReader = new InfoReader( // NOPMD
							infoXml);
					infoXml.close();
					buffer.append("<p><b>").append(infoReader.getName()) //$NON-NLS-1$
							.append("</b>"); //$NON-NLS-1$
					buffer.append(HTML_BR).append(infoReader.getDescription());
					buffer.append(HTML_BR).append(infoReader.getUrl());
					buffer.append(HTML_BR).append(infoReader.getLicense());
					buffer.append("</p>"); //$NON-NLS-1$

				} catch (Exception e) { // NOPMD
					Logger.error(
							Settings.getI18n()
									.tr("LicenseControl.infoFileParseError", infos[0].getName()), e); //$NON-NLS-1$
				}
			}
			buffer.append("</form>"); //$NON-NLS-1$

			libList = new FormTextPart(buffer.toString());
		}
		return libList;
	}
}
