/*
 * Copyright 2014 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.List;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.Currency;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.CurrencyConverterYahoo;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.frankmuenster.jameica.depot.gui.menu.WertpapierListeMenu;
import de.frankmuenster.jameica.depot.gui.parts.column.ExchangeInfo;
import de.frankmuenster.jameica.depot.gui.parts.column.WertpapierInStandardWaehrung;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.formatter.DateFormatter;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.logging.Logger;

/**
 * @author munstfr
 * 
 */
public class WertpapierListePart extends TablePart implements Part {

	/**
	 * W&auml;hrungspalte anzeigen
	 */
	private transient Boolean showWaehrSpalte;

	/**
	 * Splate f&uuml; die Standardw&auml;hrung ???
	 */
	private transient WertpapierInStandardWaehrung wpInStdWaehrung;

	/**
	 * Die Summe aller WP.
	 */
	private transient Double gesamtSumme;

	/**
	 * Konstruktor mit Angabe der Action beim Doppelklick.
	 * 
	 * @param action
	 * @throws RemoteException
	 */
	public WertpapierListePart(final Action action) throws RemoteException {
		this(init(), action);
	}

	/**
	 * Konstruktor mit Angabe der Liste und der Action beim Doppelklick.
	 * 
	 * @param list
	 * @param action
	 * @throws RemoteException
	 */
	public WertpapierListePart(final DBIterator<?> list, final Action action) throws RemoteException {

		super(list, action);

		// Berechne Gesamtsumme
		BigDecimal sumUp = new BigDecimal("0.0"); //$NON-NLS-1$
		final CurrencyConverterYahoo ccy = CurrencyConverterYahoo.getInstance();
		while (list.hasNext()) {
			final Wertpapier wertpapier = (Wertpapier) list.next();
			Logger.debug(Settings.getI18n().tr("WertpapierListePart.logInfoStockValue", //$NON-NLS-1$
					wertpapier.getName(), Double.toString(wertpapier.getGesamtWert())));
			double gesamtWert = wertpapier.getGesamtWert();
			if (!Currency.EUR.toString().equals(wertpapier.getCurrency())) {
				try {
					final double wpWertInEur = ccy.exchangeRate(wertpapier.getCurrency(), "EUR"); //$NON-NLS-1$
					gesamtWert = gesamtWert * wpWertInEur;
				} catch (YahooFinanceException e) {
					gesamtWert = 0.0d;
					Logger.error(Settings.getI18n().tr("WertpapierListePart.getExchangeRateError", //$NON-NLS-1$
							e.getMessage()));
				}
			}
			sumUp = sumUp.add(new BigDecimal(gesamtWert)); //NOPMD
		}
		
		gesamtSumme = sumUp.doubleValue();

		setRememberColWidths(true);
		setRememberOrder(true);
		setRememberState(true);

		// Whrungsspalte?
		showWaehrSpalte = showWaehrungsSpalte(Settings.getDepotWaehrung(), list);

		final DateFormatter daf = new DateFormatter();

		// addColumn("ID", "id");
		addColumn(Settings.getI18n().tr("WertpapierListePart.colName"), "name"); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(Settings.getI18n().tr("WertpapierListePart.colSymbole"), "symbol"); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(Settings.getI18n().tr("WertpapierListePart.number"), "anzahl"); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(Settings.getI18n().tr("WertpapierListePart.rateDate"), "lastTradeDate", daf); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(Settings.getI18n().tr("WertpapierListePart.rate"), "lastTradePriceOnly", //$NON-NLS-1$ //$NON-NLS-2$
				FormatterUtils.CURRENCY_FORMAT.get());
		addColumn(Settings.getI18n().tr("WertpapierListePart.currency"), "currency"); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(Settings.getI18n().tr("WertpapierListePart.value"), "gesamtWert", //$NON-NLS-1$ //$NON-NLS-2$
				FormatterUtils.CURRENCY_FORMAT.get());
		if (showWaehrSpalte) {
			wpInStdWaehrung = new WertpapierInStandardWaehrung();
			addColumn(wpInStdWaehrung);
		}
		addColumn(Settings.getI18n().tr("WertpapierListePart.comment"), "comment"); //$NON-NLS-1$ //$NON-NLS-2$

		// Kontext Men hinzufgen
		setContextMenu(new WertpapierListeMenu());

		this.setSummary(true);

	}

	/**
	 * Liefert <i>true</i> falls die W&auml;hrungsspalte angezeigt werden muss,
	 * sonst <i>false</i>.
	 * 
	 * @return <i>true</i> falls die W&auml;hrungsspalte angezeigt werden muss,
	 *         sonst <i>false</i>
	 */
	public boolean isShowWaehrungsSpalte() {
		return showWaehrSpalte;
	}

	/**
	 * Checked, ob die Currency-Spalte angezeigt werden muss.
	 * 
	 * @param waehrung
	 * @param list
	 * @return <i>true</i> falls die W&auml;hrungsspalte angezeigt werden muss,
	 *         sonst <i>false</i>
	 * @throws RemoteException
	 */
	private boolean showWaehrungsSpalte(final String waehrung, final DBIterator<?> list) throws RemoteException {

		boolean mustShow = false;

		list.begin();
		while (list.hasNext()) {
			final Wertpapier wertpapier = (Wertpapier) list.next();
			if (!waehrung.equals(wertpapier.getCurrency())) {
				// Andere Whrung gefunden, also anzeigen.
				mustShow = true;
				break;
			}
		}
		// Alles in einer Waehrung, also nicht anzeigen.
		return mustShow;
	}

	/**
	 * Liefert Umtauschraten von WP, die nicht in Standardw&auml;hrung sind.
	 * 
	 * @return Liste vom Typ <i>ExchangeInfo</i> mit den Umtauschraten.
	 */
	public List<ExchangeInfo> getExchangeInfos() {
		return wpInStdWaehrung.getExchangeInfos();
	}

	/**
	 * Liefert die GesamtSumme aller WP.
	 * 
	 * @return die gesamtsumme
	 */
	public Double getGesamtSumme() {
		return gesamtSumme;
	}

	/**
	 * Liefert die sortierte Wertpapierliste. Das ist nur deshalb in einer extra
	 * Funktion ausgegliedert, weil der Aufruf des super-Konstruktors zwingend
	 * in der ersten Zeile erfolgen muss.
	 * 
	 * @return Liste mit dem WP.
	 * @throws RemoteException
	 */
	private static DBIterator<?> init() throws RemoteException {

		final JaDepotDBService service = (JaDepotDBService) Settings.getDBService();
		final DBIterator<?> list = service.createList(Wertpapier.class);
		list.setOrder("ORDER BY name asc"); //$NON-NLS-1$
		return list;
	}

	/**
	 * @see TablePart#getSummary()
	 */
	@Override
	protected String getSummary() {

		String summery = super.getSummary();
		summery = Settings.getI18n().tr("WertpapierListePart.totalValue", //$NON-NLS-1$
				new String[] { FormatterUtils.DECIMAL_FORMAT.get().format(gesamtSumme), Settings.CURRENCY });

		return summery;

	}
}
