package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * Part zum Anzeigen der Wertpapierperformanz. dazu geh&ouml;ren die Anzahl der
 * K&auml;ufe, Verk&auml;ufe, Zusatzkosten und die Ergebnissaufstellung
 * &uuml;ber alle Kosten und ertr&auml;ge.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class WertpapierPerformanzPart implements Part {

	/**
	 * Nicht verf&uuml;gbar String
	 */
	private final static String NOT_AVAILABLE = Settings.getI18n().tr(
			"WertpapierPerformanzPart.notAvailable"); //$NON-NLS-1$

	/**
	 * Das Wertpapier des Performanz angezeigt werden soll
	 */
	private final transient Wertpapier wertpapier;

	/**
	 * Gesamtwert aller Papiere-Label
	 */
	private transient LabelInputExtended gesamtWert;

	/**
	 * Kosten aller Verk&auml;ufe-Label
	 */
	private transient LabelInputExtended verkauf;

	/**
	 * Einnahmen aller K&auml;ufe-Label
	 */
	private transient LabelInputExtended kauf;

	/**
	 * Gesamtwert aller zus&auml;ztlichen Kosten-Label
	 */
	private transient LabelInputExtended kosten;

	/**
	 * Summe aller Einnahmen/Kosten-Label
	 */
	private transient LabelInputExtended ergebnis;

	/**
	 * Konstruktor mit Angabe des Wertpapiers.
	 * 
	 * @param wertpapier
	 */
	public WertpapierPerformanzPart(final Wertpapier wertpapier) {
		this.wertpapier = wertpapier;
	}

	/**
	 * @see Part#paint(Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		final SimpleContainer simpleContainer = new SimpleContainer(parent,
				false, 2);

		if (wertpapier != null) {

			// Gesamtwert
			this.gesamtWert = new LabelInputExtended("", Settings.getI18n().tr( //$NON-NLS-1$
					Settings.getI18n().tr(
							"WertpapierPerformanzPart.lblgesamtWertTooltip"))); //$NON-NLS-1$

			// Einnahmen aus Verkufen
			this.verkauf = new LabelInputExtended("", Settings.getI18n().tr( //$NON-NLS-1$
					Settings.getI18n().tr(
							"WertpapierPerformanzPart.lblVerkaufTooltip"))); //$NON-NLS-1$
			this.verkauf.setColor(Color.SUCCESS);

			// Ausgaben fr Kufe
			this.kauf = new LabelInputExtended("", Settings.getI18n().tr( //$NON-NLS-1$
					Settings.getI18n().tr(
							"WertpapierPerformanzPart.lblKaufTooltip"))); //$NON-NLS-1$
			this.kauf.setColor(Color.ERROR);

			// Summe aller zustzlichen Kosten
			this.kosten = new LabelInputExtended("", Settings.getI18n().tr( //$NON-NLS-1$
					Settings.getI18n().tr(
							"WertpapierPerformanzPart.lblKostenTooltip"))); //$NON-NLS-1$
			this.kosten.setColor(Color.ERROR);

			// Das Gesamtergbnis
			this.ergebnis = new LabelInputExtended("", Settings.getI18n().tr( //$NON-NLS-1$
					Settings.getI18n().tr(
							"WertpapierPerformanzPart.lblErgebnisTooltip"))); //$NON-NLS-1$

			// Felder aktualisieren
			update();

			// Jetzt alle Felder in den Container einfgen
			simpleContainer.addLabelPair(
					Settings.getI18n().tr(
							"WertpapierPerformanzPart.lblgesamtWert"), //$NON-NLS-1$
					gesamtWert);
			simpleContainer
					.addLabelPair(
							Settings.getI18n().tr(
									"WertpapierPerformanzPart.lblVerkauf"), verkauf); //$NON-NLS-1$
			simpleContainer
					.addLabelPair(
							Settings.getI18n().tr(
									"WertpapierPerformanzPart.lblKauf"), kauf); //$NON-NLS-1$
			simpleContainer
					.addLabelPair(
							Settings.getI18n().tr(
									"WertpapierPerformanzPart.lblKosten"), kosten); //$NON-NLS-1$
			simpleContainer.addLabelPair(
					Settings.getI18n().tr(
							"WertpapierPerformanzPart.lblErgebnis"), ergebnis); //$NON-NLS-1$
		}

	}

	/**
	 * Aktualisiert die Werte aus der Datenbank
	 */
	public void update() {

		String currency;
		try {
			currency = wertpapier.getCurrency();
		} catch (RemoteException e) {
			// Hmm, keine Whrung, also nehmen wir mal Euro an
			currency = "EUR"; //$NON-NLS-1$
		}

		// Gesamtwert
		try {
			final double bestand = wertpapier.getGesamtWert();
			this.gesamtWert.setValue(FormatterUtils.DECIMAL_FORMAT.get()
					.format(bestand) + " " + currency); //$NON-NLS-1$
		} catch (RemoteException e) {
			this.gesamtWert.setValue(Settings.getI18n().tr(NOT_AVAILABLE));
		}

		// Einnahmen aus Verkufen
		try {
			final double verkauf = wertpapier.getVerkauf();
			this.verkauf.setValue(FormatterUtils.DECIMAL_FORMAT.get().format(
					verkauf)
					+ " " + currency); //$NON-NLS-1$
		} catch (RemoteException e) {
			this.verkauf.setValue(Settings.getI18n().tr(NOT_AVAILABLE));
		}

		// Ausgaben fr Kufe
		try {
			final double kauf = wertpapier.getKauf();
			this.kauf.setValue(FormatterUtils.DECIMAL_FORMAT.get().format(kauf)
					+ " " + currency); //$NON-NLS-1$
		} catch (RemoteException e) {
			this.kauf.setValue(Settings.getI18n().tr(NOT_AVAILABLE));
		}

		// Summe aller zustzlichen Kosten
		try {
			final double kost = wertpapier.getAdditionalCosts();
			this.kosten.setValue(FormatterUtils.DECIMAL_FORMAT.get().format(
					kost)
					+ " " + currency); //$NON-NLS-1$
			this.kosten.setColor(Color.ERROR);
		} catch (RemoteException e) {
			this.kosten.setValue(Settings.getI18n().tr(NOT_AVAILABLE));
		}

		try {
			final double bestand = wertpapier.getGesamtWert();
			final double realisiert = wertpapier.getVerkauf();
			final double kosten = wertpapier.getKauf();
			final double zusatzKosten = wertpapier.getAdditionalCosts();
			final double erg = bestand + realisiert - kosten + zusatzKosten;
			this.ergebnis.setValue(FormatterUtils.DECIMAL_FORMAT.get().format(
					erg)
					+ " " + currency); //$NON-NLS-1$
			if (erg >= 0) {
				this.ergebnis.setColor(Color.SUCCESS);
			} else {
				this.ergebnis.setColor(Color.ERROR);
			}
		} catch (RemoteException e) {
			this.ergebnis.setValue(NOT_AVAILABLE);
			this.ergebnis.setColor(Color.BACKGROUND);
		}
	}
}
