/*
 * Copyright 2014 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.column;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.logging.Logger;

/**
 * Zusatzkostenspalte in Standardw&auml;hrung f&uuml;r die Order-Tabelle
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class ZusatzkostenInStandardWaehrung extends
		AbstractColumnInStandardWaehrung {

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = -4356082272860968994L;

	/**
	 * Konstruktor mit Angabe eines Formatters
	 * 
	 * @param formatter
	 */
	public ZusatzkostenInStandardWaehrung(final Formatter formatter) {
		this(false, formatter);
	}

	/**
	 * Konstruktor mit Angabe des useOrderDate Value. Mit der oderDate kann
	 * angegeben werden, ob die Umtauschrate zum Zeitpunkt der Order oder die
	 * aktuellste zur Verf&uuml;gung stehendes angezeigt werden soll.
	 * 
	 * @param useOrderDate
	 */
	public ZusatzkostenInStandardWaehrung(final Boolean useOrderDate) {
		this(useOrderDate, null);
	}

	/**
	 * Konstruktor mit Angabe eines Formatters und dem useOrderDate Value. Mit
	 * der oderDate kann angegeben werden, ob die Umtauschrate zum Zeitpunkt der
	 * Order oder die aktuellste zur Verf&uuml;gung stehendes angezeigt werden
	 * soll.
	 * 
	 * @param useOrderDate
	 * @param formatter
	 */
	public ZusatzkostenInStandardWaehrung(final Boolean useOrderDate,
			final Formatter formatter) {
		super(useOrderDate, formatter);
	}

	/**
	 * @see de.willuhn.jameica.gui.parts.Column#getFormattedValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	@Override
	public String getFormattedValue(final Object objectId, final Object context) {
		String display = StringUtils.EMPTY;

		if (context instanceof OrderHistory) {
			final OrderHistory orderHistory = (OrderHistory) context;
			double betrag;
			try {
				betrag = orderHistory.getAdditionalCosts();
				if (isUseOrderDate()) {
					display = getFormattedValue(betrag, orderHistory.getAktie()
							.getCurrency(), orderHistory.getOrderDate());
				} else {
					display = getFormattedValue(betrag, orderHistory.getAktie()
							.getCurrency(), null);
				}
			} catch (RemoteException e) {
				Logger.error(
						Settings.getI18n().tr(
								"WertpapierInStandardWaehrung.formatError", //$NON-NLS-1$
								(String) objectId, (String) context), e);
				GUI.getStatusBar().setErrorText(
						Settings.getI18n().tr(
								"WertpapierInStandardWaehrung.userMsg")); //$NON-NLS-1$
			}
		}
		return display;
	}

}
