/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.grafiken;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListMap;

/**
 * Ein TreeMap Objekt f&uuml;r Kusdaten, bestehend aus einem datum und einem
 * Wert.
 * 
 * @author munstfr
 * @version 1.0
 */
public class KursDatenAsMap {

	/**
	 * Das interen TreeMap Objekt
	 */
	private transient final Map<Date, Double> valueMap = new ConcurrentSkipListMap<Date, Double>(); // NOPMD 

	/**
	 * Standardkonstruktor
	 */
	public KursDatenAsMap() {
		// Absichlich leer
	}

	/**
	 * F&uuml;gt ein Wertepaar hinzu.
	 * 
	 * @param date
	 * @param value
	 */
	public void add(final Date date, final Double value) {
		this.valueMap.put(date, value);
	}

	/**
	 * L&ouml;scht alle Werte aus der Map.
	 */
	public void clear() {
		this.valueMap.clear();
	}

	/**
	 * Liefert eine Liste aller Daten aus der Map.
	 * 
	 * @return Liste aller Daten
	 */
	public List<Date> getDateList() {
		final List<Date> dateList = new ArrayList<Date>();

		final Set<Entry<Date, Double>> set2 = valueMap.entrySet();
		final Iterator<Entry<Date, Double>> iterator2 = set2.iterator();
		while (iterator2.hasNext()) {
			final Map.Entry<java.util.Date, java.lang.Double> entry = iterator2
					.next();
			dateList.add(entry.getKey());
		}
		return dateList;
	}

	/**
	 * Liefert eine Liste aller Werte aus der Map.
	 * 
	 * @return Liste aller Werte
	 */
	public List<Double> getValueList() {
		final List<Double> valueList = new ArrayList<Double>();

		final Set<Entry<Date, Double>> set2 = valueMap.entrySet();
		final Iterator<Entry<Date, Double>> iterator2 = set2.iterator();
		while (iterator2.hasNext()) {
			final Map.Entry<java.util.Date, java.lang.Double> entry = iterator2
					.next();
			valueList.add(entry.getValue());
		}
		return valueList;
	}

}
