/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.grafiken.kursgrafik;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.PlotEntity;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.CandlestickRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.OHLCDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.experimental.chart.swt.ChartComposite;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.Part;

/**
 * Die Chartgrafik f&uuml;r die WertpapierDeatil View.
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings("PMD.ExcessiveImports")
public class KursGrafik extends AbstractGrafik implements ChartMouseListener, Part {

	/**
	 * Unser Composite in dem wir malen
	 */
	private transient Composite body;

	/**
	 * Das ChartComposite
	 */
	private transient ChartComposite frame;

	/**
	 * Standardkonstruktor
	 */
	public KursGrafik() {
		super();
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		body = new Composite(parent, SWT.NONE);
		body.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		body.setLayout(new FillLayout());

		reload();
	}

	/**
	 * 
	 * @see de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik#reload()
	 */
	@Override
	public void reload() {

		JFreeChart chart;

		for (final Control element : body.getChildren()) {
			element.dispose();
		}

		if (showCandle) {
			chart = getCandleStickChart(getDatasetCandleStick());
		} else if (showDax) {
			chart = getDaxChart(getDatasetKursDaxVergleich());
		} else {
			chart = getKursChart(getDataSetKurs());
		}

		final XYPlot mainPlot = (XYPlot) chart.getPlot();

		mainPlot.setDomainCrosshairVisible(true);
		mainPlot.setDomainCrosshairLockedOnData(true);
		mainPlot.setRangeCrosshairVisible(true);

		// Datumanzeige anpassen
		final DateAxis dateAxis = ((DateAxis) mainPlot.getDomainAxis());
		final Date min = dateAxis.getMinimumDate();
		final Date max = dateAxis.getMaximumDate();
		final long diff = max.getTime() - min.getTime();
		final long oneYear = 1000l * 60 * 60 * 24 * 365;

		SimpleDateFormat dateFormat;
		if (diff > oneYear) {
			dateFormat = new SimpleDateFormat("ww-yy", Locale.GERMAN); //$NON-NLS-1$
			dateAxis.setLabel(Settings.getI18n().tr("KursGrafik.weekOfYear")); //$NON-NLS-1$
		} else {
			dateFormat = new SimpleDateFormat("dd.MM", Locale.GERMAN); //$NON-NLS-1$
			final Calendar calendar = new GregorianCalendar();
			calendar.setTime(min);
			final int yearMin = calendar.get(Calendar.YEAR);
			calendar.setTime(max);
			final int yearMax = calendar.get(Calendar.YEAR);
			if (yearMin == yearMax) {
				dateAxis.setLabel(Settings.getI18n().tr("KursGrafik.dayOfMonthInYear") //$NON-NLS-1$
						+ Integer.toString(yearMin));
			} else {
				dateAxis.setLabel(Settings.getI18n().tr("KursGrafik.dayOfMonthInYears", //$NON-NLS-1$
						Integer.toString(yearMin), Integer.toString(yearMax)));
			}
		}
		dateAxis.setDateFormatOverride(dateFormat);

		chart.setTitle(StringUtils.EMPTY);

		// Hauptlegende entfernen
		chart.removeLegend();

		this.chart = chart;

		frame = new ChartComposite(body, SWT.NONE, chart, true);
		frame.addChartMouseListener(this);

		body.layout(true, true);

	}

	/**
	 * Liefert das KursChart
	 * 
	 * @param dataset
	 * @return das KursChart
	 */
	private JFreeChart getKursChart(final XYDataset dataset) {

		final JFreeChart chart = ChartFactory.createTimeSeriesChart(Settings.getI18n().tr("KursGrafik.chartTitle"), //$NON-NLS-1$
				Settings.getI18n().tr("KursGrafik.chartTimeAxisLabel"), //$NON-NLS-1$
				Settings.getI18n().tr("KursGrafik.chartValueAxisLabel"), //$NON-NLS-1$
				dataset, true, true, false);

		// Set chart background
		chart.setBackgroundPaint(Color.white);

		// Set a few custom plot features
		final XYPlot plot = (XYPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinesVisible(true);
		plot.setDomainGridlinePaint(Color.lightGray);
		plot.setRangeGridlinePaint(Color.lightGray);
		((NumberAxis) plot.getRangeAxis()).setAutoRangeIncludesZero(false);
		((NumberAxis) plot.getRangeAxis()).setLabelFont(FONT_AXIS);

		// Skip week-ends on the date axis
		((DateAxis) plot.getDomainAxis()).setDateFormatOverride(new SimpleDateFormat("dd.MM", Locale.GERMAN)); //$NON-NLS-1$
		((DateAxis) plot.getDomainAxis()).setLabelFont(FONT_AXIS);

		// Nice tool tip format
		final XYItemRenderer renderer = chart.getXYPlot().getRenderer();
		final StandardXYToolTipGenerator toolTipGenerator = new StandardXYToolTipGenerator(
				StandardXYToolTipGenerator.DEFAULT_TOOL_TIP_FORMAT, FormatterUtils.GERMAN_DATE_FORMAT.get(),
				new DecimalFormat("0.00 EUR")); //$NON-NLS-1$
		// renderer.setToolTipGenerator(toolTipGenerator);
		renderer.setBaseToolTipGenerator(toolTipGenerator);

		return chart;
	}

	/**
	 * Liefert das DAX Chart
	 * 
	 * @param dataset
	 * @return das DAX Chart
	 */
	private JFreeChart getDaxChart(final TimeSeriesCollection dataset) {

		final JFreeChart chart = ChartFactory.createTimeSeriesChart(Settings.getI18n().tr("KursGrafik.chartTitle"), //$NON-NLS-1$
				Settings.getI18n().tr("KursGrafik.chartTimeAxisLabel"), //$NON-NLS-1$
				Settings.getI18n().tr("KursGrafik.chartProcentageAxisLabel"), //$NON-NLS-1$
				dataset, true, true, false);

		// Set chart background
		chart.setBackgroundPaint(Color.white);

		// Set a few custom plot features
		final XYPlot plot = (XYPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.WHITE); // light yellow = new
												// Color(0xffffe0)
		plot.setDomainGridlinesVisible(true);
		plot.setDomainGridlinePaint(Color.lightGray);
		plot.setRangeGridlinePaint(Color.lightGray);
		((NumberAxis) plot.getRangeAxis()).setAutoRangeIncludesZero(false);
		((NumberAxis) plot.getRangeAxis()).setLabelFont(FONT_AXIS);

		// Skip week-ends on the date axis --> doesn't work with candle stick
		((DateAxis) plot.getDomainAxis()).setDateFormatOverride(new SimpleDateFormat("dd.MM", Locale.GERMAN)); //$NON-NLS-1$
		((DateAxis) plot.getDomainAxis()).setLabelFont(FONT_AXIS);

		// Nice tool tip format
		final XYItemRenderer renderer = chart.getXYPlot().getRenderer();
		final StandardXYToolTipGenerator toolTipGenerator = new StandardXYToolTipGenerator(
				StandardXYToolTipGenerator.DEFAULT_TOOL_TIP_FORMAT, FormatterUtils.GERMAN_DATE_FORMAT.get(),
				new DecimalFormat("0.00 EUR")); //$NON-NLS-1$
		// renderer.setToolTipGenerator(toolTipGenerator);
		renderer.setBaseToolTipGenerator(toolTipGenerator);

		// Legende hinzufgen
		final LegendTitle legendTitle = new LegendTitle(plot);
		legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
		legendTitle.setBackgroundPaint(new Color(200, 200, 255, 100));
		legendTitle.setFrame(new BlockBorder(Color.white));
		legendTitle.setPosition(RectangleEdge.BOTTOM);
		final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
		legendTitle.setItemLabelPadding(padding);
		final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(1.0, 0.00, legendTitle,
				RectangleAnchor.BOTTOM_RIGHT);

		titleAnnotation.setMaxWidth(0.48);
		plot.addAnnotation(titleAnnotation);

		return chart;
	}

	/**
	 * Liefert das Chart als Kerzengrafik.
	 * 
	 * @param dataset
	 * @return das Chart als Kerzengrafik
	 */
	private JFreeChart getCandleStickChart(final OHLCDataset dataset) {

		final JFreeChart chart = ChartFactory.createCandlestickChart(Settings.getI18n().tr("KursGrafik.chartTitle"), //$NON-NLS-1$
				Settings.getI18n().tr("KursGrafik.chartTimeAxisLabel"), //$NON-NLS-1$
				Settings.getI18n().tr("KursGrafik.chartValueAxisLabel"), //$NON-NLS-1$
				dataset, true);

		// Set chart background
		chart.setBackgroundPaint(Color.white);

		// Set a few custom plot features
		final XYPlot plot = (XYPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinesVisible(true);
		plot.setDomainGridlinePaint(Color.lightGray);
		plot.setRangeGridlinePaint(Color.lightGray);
		((NumberAxis) plot.getRangeAxis()).setAutoRangeIncludesZero(false);
		((NumberAxis) plot.getRangeAxis()).setLabelFont(FONT_AXIS);

		// Skip week-ends on the date axis --> doesn't work with candle stick
		((DateAxis) plot.getDomainAxis()).setLabelFont(FONT_AXIS);

		// Skip week-ends on the date axis
		// final DateAxis dateAxis = (DateAxis) plot.getDomainAxis();

		// Now volume drawn
		final CandlestickRenderer renderer = ((CandlestickRenderer) plot.getRenderer());
		renderer.setDrawVolume(true);

		return chart;
	}

	/**
	 * @see org.jfree.chart.ChartMouseListener#chartMouseClicked(org.jfree.chart.ChartMouseEvent)
	 */
	@Override
	public void chartMouseClicked(final ChartMouseEvent arg0) {
		final double xValue = ((XYPlot) this.chart.getPlot()).getDomainCrosshairValue();
		final double yValue = ((XYPlot) this.chart.getPlot()).getRangeCrosshairValue();
		reload();
		((XYPlot) this.chart.getPlot()).setDomainCrosshairValue(xValue);
		((XYPlot) this.chart.getPlot()).setRangeCrosshairValue(yValue);

	}

	/**
	 * @see org.jfree.chart.ChartMouseListener#chartMouseMoved(org.jfree.chart.ChartMouseEvent)
	 */
	@Override
	public void chartMouseMoved(final ChartMouseEvent event) {

		final XYPlot cdPlot = (XYPlot) this.chart.getPlot();
		final ChartEntity entity = event.getEntity();

		if (entity instanceof PlotEntity || entity instanceof XYItemEntity) {

			if (!cdPlot.isDomainCrosshairVisible()) {
				cdPlot.setDomainCrosshairVisible(true);
				cdPlot.setRangeCrosshairVisible(true);
			}
			final int mouseX = event.getTrigger().getX();
			final int mouseY = event.getTrigger().getY();
			final Point mousePoint = new Point(mouseX, mouseY);

			// convert the Java2D coordinate to axis coordinates...
			// final XYPlot cdPlot = (XYPlot) this.chart.getPlot();
			final ChartRenderingInfo chartInfo = this.frame.getChartRenderingInfo();
			final Point2D java2DPoint = this.frame.translateScreenToJava2D(mousePoint);
			final PlotRenderingInfo plotInfo = chartInfo.getPlotInfo();

			// all subplots have the domain crosshair
			// the x coordinate is the same for all subplots
			final Rectangle2D dataArea = plotInfo.getDataArea();
			// Rectangle2D panelArea = plotInfo.getDataArea();
			final double x2DValue = cdPlot.getDomainAxis().java2DToValue(java2DPoint.getX(), dataArea,
					cdPlot.getDomainAxisEdge());

			final org.eclipse.swt.graphics.Rectangle rect = this.frame.getScreenDataArea(mouseX, mouseY);
			final Rectangle2D panelArea = new Rectangle2D.Double();
			panelArea.setRect(rect.x, rect.y, rect.width, rect.height);

			final double y2DValue = cdPlot.getRangeAxis().java2DToValue(mousePoint.getY(), panelArea,
					cdPlot.getRangeAxisEdge());

			cdPlot.setDomainCrosshairValue(x2DValue, true);
			cdPlot.setRangeCrosshairValue(y2DValue, true);
		} else {
			cdPlot.setDomainCrosshairVisible(false);
			cdPlot.setRangeCrosshairVisible(false);
		}

	}

}
