/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.layout.GridData;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.controller.OrderController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
public class OrderView extends AbstractView {

	/**
	 * Der OrderController
	 */
	private transient OrderController control;

	/**
	 * Standardkonstruktor
	 */
	public OrderView() {
		super();
	}

	/**
	 * 
	 * @see de.willuhn.jameica.gui.AbstractView#bind()
	 */
	@Override
	public void bind() throws RemoteException {

		GUI.getView().setTitle(Settings.getI18n().tr("OrderView.newOrder")); //$NON-NLS-1$

		control = new OrderController(this);

		// layout with 2 columns
		final ColumnLayout columns = new ColumnLayout(getParent(), 2);

		final Container links = new SimpleContainer(columns.getComposite());
		// Suchfenster

		if (control.isNeuesWertpapier()) {
			GUI.getView().setTitle(Settings.getI18n().tr("OrderView.newStock")); //$NON-NLS-1$

			// Anzeigen nur, wenn neues Wertpapier
			links.addHeadline(Settings.getI18n().tr("OrderView.searchStock")); //$NON-NLS-1$
			links.addPart(control.getSymbolSearchPart());
		}

		// right side
		final Container right = new SimpleContainer(columns.getComposite(),
				true);
		final GridData gridData = (GridData) right.getComposite()
				.getLayoutData();
		gridData.minimumWidth = 250;
		right.addHeadline(Settings.getI18n().tr("OrderView.description")); //$NON-NLS-1$
		right.addInput(control.getComment());
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);

		// left lower
		links.addHeadline(Settings.getI18n().tr("OrderView.symboleName")); //$NON-NLS-1$
		links.addLabelPair(
				Settings.getI18n().tr("OrderView.symbole"), control.getSymbol()); //$NON-NLS-1$
		links.addLabelPair(
				Settings.getI18n().tr("OrderView.name"), control.getName()); //$NON-NLS-1$
		links.addText(StringUtils.EMPTY, true);

		links.addHeadline(Settings.getI18n().tr("OrderView.DateAndNumber")); //$NON-NLS-1$
		final ColumnLayout dateLay = new ColumnLayout(links.getComposite(), 1);
		final Container date = new SimpleContainer(dateLay.getComposite(),
				false, 4);
		date.addLabelPair(
				Settings.getI18n().tr("OrderView.Date"), control.getOrderDate()); //$NON-NLS-1$
		date.addText(
				"                                                        ", //$NON-NLS-1$
				false);
		date.addLabelPair(
				Settings.getI18n().tr("OrderView.number"), control.getVolume()); //$NON-NLS-1$
		links.addText(StringUtils.EMPTY, true);

		links.addHeadline(Settings.getI18n().tr("OrderView.prices")); //$NON-NLS-1$
		final ColumnLayout priceLay = new ColumnLayout(links.getComposite(), 1);
		final Container price = new SimpleContainer(priceLay.getComposite(),
				false, 6);
		price.addLabelPair(
				Settings.getI18n().tr("OrderView.price"), control.getPrice()); //$NON-NLS-1$
		price.addInput(control.getStockCurrency());
		price.addPart(control.getPriceInMainCurrency());
		links.addText(StringUtils.EMPTY, true);

		links.addHeadline(Settings.getI18n().tr("OrderView.additionalCosts")); //$NON-NLS-1$
		final ColumnLayout addPriceLay = new ColumnLayout(links.getComposite(),
				1);
		final Container addPrice = new SimpleContainer(
				addPriceLay.getComposite(), false, 6);
		addPrice.addLabelPair(Settings.getI18n().tr("OrderView.costs"), //$NON-NLS-1$
				control.getAdditionalCosts());
		addPrice.addInput(control.getAdditionalCostsCurrency());
		addPrice.addPart(control.getAdditionalCostsInMainCurrency());
		// Jetzt schon anzeigen
		control.updateView();

		// Speicher Button einfgen
		final ButtonArea buttons = new ButtonArea();
		final Button store = control.getStore();
		final Button cancel = control.getCancel();

		buttons.addButton(store);
		buttons.addButton(cancel);

		buttons.paint(getParent());

	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#unbind()
	 */
	@Override
	public void unbind() throws ApplicationException {
		super.unbind();
		control = null; // NOPMD
	}
}
