/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.kursanalyse.BollingBaender; // NOPMD

import java.util.Date;
import java.util.LinkedList;
import java.util.Queue;

import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * Bolling B&auml;nder
 * 
 * @author munstfr
 * @version 1.0
 */
public class BollingBaender extends AbstractKursAnalyse<Integer, KursDaten[]> {

	/**
	 * Methodenname
	 */
	private final static String NAME = "BOLLING"; //$NON-NLS-1$

	/**
	 * Bollig-B&auml;nder mit Average 10
	 */
	private final static int BOLLIG_10 = 10;

	/**
	 * Bollig-B&auml;nder mit Average 50
	 */
	private final static int BOLLIG_50 = 50;

	/**
	 * Tempor&auml;rge Queue zum Zwischenspeichern von Werten
	 */
	private final transient Queue<Double> temp = new LinkedList<Double>();

	/**
	 * Summe
	 */
	private transient double sum;

	/**
	 * Daten der Bollig Mittellinie (Trigger)
	 */
	private transient KursDaten bollingMitte;

	/**
	 * Daten der oberen Bolliglinie
	 */
	private transient KursDaten bollingOben;

	/**
	 * Daten der unteren Bolliglinie
	 */
	private transient KursDaten bollingUnten;

	/**
	 * Standardkonstruktor, ruft nur super auf.
	 */
	public BollingBaender() {
		super();
	}

	/**
	 * Konstruktor mit Angabe der Kursdaten und der zu benutzen Parameter.
	 * 
	 * @param wertpapierHistory
	 *            dir Kursdaten
	 * @param parameter
	 *            die Paramater
	 */
	public BollingBaender(final WertpapierHistory wertpapierHistory,
			final KursAnalyseParameter<Integer> parameter) {
		super(wertpapierHistory, parameter);
	}

	/**
	 * Liefert den Methodennamen
	 * 
	 * @return der Methodennamen
	 */
	public static String getMethodName() {
		return NAME;
	}

	/**
	 * Liefert die AddAction
	 * 
	 * @return die BollingBaenderHandleAddAction
	 */
	public static Action getHandleAction() {
		return new BollingBaenderHandleAddAction();
	}

	/**
	 * @see AbstractKursAnalyse#getResult(Date, Date)
	 */
	@Override
	public KursDaten[] getResult(final Date fromDate, final Date toDate) {

		if (bollingMitte == null) {
			calculate();
		}

		// Ergbniss zusammenkrzen auf Zeitangebe
		final KursDaten bbMitte = new KursDaten();
		final KursDaten bbOben = new KursDaten();
		final KursDaten bbUnten = new KursDaten();

		for (int i = 0; i < bollingMitte.size(); i++) {
			final DateValuePair bbMitteValue = bollingMitte.get(i);
			if (fromDate.getTime() < bbMitteValue.getDate().getTime()
					&& bbMitteValue.getDate().getTime() < toDate.getTime()) {
				final DateValuePair bbObenValue = bollingOben.get(i);
				final DateValuePair bbUntenValue = bollingUnten.get(i);
				bbMitte.add(bbMitteValue);
				bbOben.add(bbObenValue);
				bbUnten.add(bbUntenValue);
			}
		}

		// nun das Ergebnis zusammenbasteln
		KursDaten[] result = new KursDaten[3];
		result[0] = bbMitte;
		result[1] = bbOben;
		result[2] = bbUnten;

		return result;

	}

	/**
	 * @see AbstractKursAnalyse#getLegendTitel ()
	 */
	@Override
	public String getLegendTitel() {
		return "Bolling [" + parameter.getValue() + "]"; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * Hier werden die Bollig-B&auml;nderdaten berechnet.
	 */
	private void calculate() {

		bollingMitte = new KursDaten();
		bollingOben = new KursDaten();
		bollingUnten = new KursDaten();

		double faktor = 2.0;
		if (parameter.getValue() == BOLLIG_10) {
			faktor = 1.9;
		} else if (parameter.getValue() == BOLLIG_50) {
			faktor = 2.1;
		}

		for (int i = 0; i < closeDaten.size(); i++) {
			final DateValuePair orgValue = closeDaten.get(i);
			add(orgValue.getValue());
			final double bollingMitteValue = getAverage();
			final double stdAbweichung = getStandardAbweichung(bollingMitteValue);
			final double bollingObenValue = bollingMitteValue
					+ (stdAbweichung * faktor);
			final double bollingUntenValue = bollingMitteValue
					- (stdAbweichung * faktor);
			if (!Double.isNaN(bollingMitteValue)) {
				bollingMitte.add(new DateValuePair(orgValue.getDate(), // NOPMD
						bollingMitteValue));
				bollingOben.add(new DateValuePair(orgValue.getDate(), // NOPMD
						bollingObenValue));
				bollingUnten.add(new DateValuePair(orgValue.getDate(), // NOPMD
						bollingUntenValue));
			}
		}

	}

	/**
	 * Berechnet die Standardabweichung des Mittelbandes f&uuml;r den angegeben
	 * Zeitraum.
	 * 
	 * @param bollingMitte
	 * @return die Standardabweichung
	 */
	private double getStandardAbweichung(final double bollingMitte) {

		Double stdAbweichung = Double.NaN;

		if (temp.size() >= parameter.getValue()) {
			double sumAbweichungen = 0.0;
			for (final double close : temp) {
				sumAbweichungen += (close - bollingMitte)
						* (close - bollingMitte);
			}
			stdAbweichung = Math.sqrt(sumAbweichungen / temp.size());
		}
		return stdAbweichung;
	}

	/**
	 * F&uuml;gt einen Wert in die Queue. Wurde die Zeitspane aus Parameter
	 * erreicht, wird der &auml;lteste Wert aus der Queue gel&ouml;scht. Die
	 * Summe sum wird ebenfalls immer aus letzetn n Werten (aus Parameter)
	 * aktualisiert.
	 * 
	 * @param wert
	 */
	private void add(final double wert) {

		sum += wert;
		temp.add(wert);
		if (temp.size() > parameter.getValue()) {
			sum -= temp.remove();
		}
	}

	/**
	 * Liefert den Durchschnitt der letzten n-Werte (wenn auch mindestens
	 * n-Werte vorhanden sind.
	 * 
	 * @return der Durchschnitt der letzten n-Werte
	 */
	private double getAverage() {

		Double durchschnitt = Double.NaN;
		if (temp.size() >= parameter.getValue()) {
			durchschnitt = sum / temp.size();
		}
		return durchschnitt;
	}

}
