/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.kursanalyse.MACD; // NOPMD 

import java.awt.Color;
import java.awt.Font;
import java.awt.Paint;
import java.util.Date;

import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SegmentedTimeline;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYBarPainter;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.time.Day;
import org.jfree.data.time.RegularTimePeriod;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.willuhn.jameica.gui.Action;
import de.willuhn.logging.Logger;

/**
 * MACD
 * 
 * @author munstfr
 * @version 1.0
 */
public class MACD extends AbstractKursAnalyse<MACDParameter, XYPlot> { // NOPMD

	/**
	 * Methodenname
	 */
	private final static String NAME = "MACD"; //$NON-NLS-1$

	/**
	 * Daten der MACD Linie
	 */
	private transient TimeSeries serieMacd;

	/**
	 * Daten der Trigger Linie
	 */
	private transient TimeSeries serieTrigger;

	/**
	 * Der der Histogrambalken
	 */
	private transient TimeSeries serieHistogram;

	/**
	 * Der MACD Renderer
	 */
	private transient XYItemRenderer rendererMACD;

	/**
	 * Der Trigger Renderer
	 */
	private transient XYItemRenderer rendererTrigger;

	/**
	 * Der Histogram Renderer
	 */
	private transient MyXYBarRenderer rendererHistogram;

	/**
	 * Liefert den Methodennamen
	 * 
	 * @return der Methodennamen
	 */
	public static String getMethodName() {
		return NAME;
	}

	/**
	 * Liefert die AddAction
	 * 
	 * @return ie MACDHandleAddAction
	 */
	public static Action getHandleAction() {
		return new MACDHandleAddAction();
	}

	/**
	 * @see AbstractKursAnalyse#getLegendTitel()
	 */
	@Override
	public String getLegendTitel() {
		return "MACD"; //$NON-NLS-1$
	}

	/**
	 * @see AbstractKursAnalyse#getResult(Date, Date)
	 */
	@Override
	public XYPlot getResult(final Date fromDate, final Date toDate) {

		if (anaG == null) {

			final DateAxis dateAxisKurs = new DateAxis(Settings.getI18n().tr(
					"MACD.axisDate")); //$NON-NLS-1$
			final NumberAxis rangeAxisKurs = new NumberAxis(getLegendTitel());
			rangeAxisKurs.setAutoRangeIncludesZero(false);
			anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
			((DateAxis) anaG.getDomainAxis()).setTimeline(SegmentedTimeline
					.newMondayThroughFridayTimeline());
			anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);

			// XYAreaRenderer rendererMACD = new XYAreaRenderer();
			rendererMACD = new StandardXYItemRenderer();
			rendererMACD.setSeriesPaint(0, Color.BLACK);

			rendererTrigger = new StandardXYItemRenderer();
			rendererTrigger.setSeriesPaint(0, Color.RED);

			rendererHistogram = new MyXYBarRenderer();
			rendererHistogram.setSeriesVisibleInLegend(0, false);
			rendererHistogram.setShadowVisible(false);
			rendererHistogram.setMargin(0.85);
			rendererHistogram.setBarPainter(new StandardXYBarPainter());

			final LegendTitle legendTitle = new LegendTitle(anaG);
			legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
			legendTitle.setBackgroundPaint(new Color(200, 200, 255, 100));
			legendTitle.setFrame(new BlockBorder(Color.white));
			legendTitle.setPosition(RectangleEdge.BOTTOM);
			final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
			legendTitle.setItemLabelPadding(padding);
			final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(
					1.0, 0.00, legendTitle, RectangleAnchor.BOTTOM_RIGHT);
			titleAnnotation.setMaxWidth(0.48);
			anaG.addAnnotation(titleAnnotation);

		}

		if (serieMacd == null) {
			callculate();
		}

		final Day fromDay = new Day(fromDate);
		final Day toDay = new Day(toDate);

		try {
			final TimeSeriesCollection tmpMacd = new TimeSeriesCollection(
					serieMacd.createCopy(fromDay, toDay));
			final TimeSeriesCollection tmpTrigger = new TimeSeriesCollection(
					serieTrigger.createCopy(fromDay, toDay));
			final TimeSeriesCollection tmpHistogram = new TimeSeriesCollection(
					serieHistogram.createCopy(fromDay, toDay));

			anaG.setDataset(0, tmpMacd);
			anaG.setDataset(1, tmpTrigger);
			anaG.setDataset(2, tmpHistogram);

			anaG.setRenderer(0, rendererMACD);
			anaG.setRenderer(1, rendererTrigger);
			anaG.setRenderer(2, rendererHistogram);

		} catch (CloneNotSupportedException e) {
			Logger.error(Settings.getI18n().tr("MACD.createTsDataError"), e); //$NON-NLS-1$
		}

		return anaG;
	}

	/**
	 * Nun mal rechnen
	 */
	private void callculate() {

		final MACDParameter para = parameter.getValue();

		serieMacd = new TimeSeries(getLegendTitel());
		serieTrigger = new TimeSeries(Settings.getI18n().tr(
				"MACD.seriesTriggerName")); //$NON-NLS-1$
		serieHistogram = new TimeSeries(Settings.getI18n().tr(
				"MACD.seriesHistogramName")); //$NON-NLS-1$

		// MACD Daten
		final KursDaten emaKurz = new KursDaten();
		final KursDaten emaLang = new KursDaten();

		double ewK;
		double ewL;

		ewL = 2.0 / (para.getEmaLang() + 1.0);
		ewK = 2.0 / (para.getEmaKurz() + 1.0);

		emaKurz.add(new DateValuePair(closeDaten.get(0).getDate(), closeDaten
				.get(0).getValue()));
		emaLang.add(new DateValuePair(closeDaten.get(0).getDate(), closeDaten
				.get(0).getValue()));

		Date date = closeDaten.get(0).getDate();
		Day day = new Day(date);
		serieMacd.add(day, 0.0);
		for (int i = 1; i < closeDaten.size(); i++) {

			final double emaKurzValue = emaKurz.get(i - 1).getValue()
					+ (ewK * (closeDaten.get(i).getValue() - emaKurz.get(i - 1)
							.getValue()));
			emaKurz.add(new DateValuePair(closeDaten.get(i).getDate(), // NOPMD
					emaKurzValue));

			final double emaLangValue = emaLang.get(i - 1).getValue()
					+ (ewL * (closeDaten.get(i).getValue() - emaLang.get(i - 1)
							.getValue()));
			emaLang.add(new DateValuePair(closeDaten.get(i).getDate(), // NOPMD
					emaLangValue));

			date = closeDaten.get(i).getDate();
			day = new Day(date); // NOPMD
			serieMacd.add(day, (emaKurzValue - emaLangValue));
		}

		// Trigger und Histogram
		final double ewS = 2.0 / (para.getEmaSignal() + 1.0);

		// Startwerte
		serieTrigger.add(serieMacd.getDataItem(0).getPeriod(), serieMacd
				.getDataItem(0).getValue());
		serieHistogram.add(serieMacd.getDataItem(0).getPeriod(), 0.0);

		for (int i = 1; i < serieMacd.getItemCount(); i++) {

			// Datum
			final RegularTimePeriod date1 = serieMacd.getDataItem(i)
					.getPeriod();

			// Trigger
			final double trigger0 = (Double) serieTrigger.getDataItem(i - 1)
					.getValue();
			final double macd1 = (Double) serieMacd.getDataItem(i).getValue();
			final double trigger1 = trigger0 + (ewS * (macd1 - trigger0));
			serieTrigger.add(date1, trigger1);

			// Histogram
			final double histogram = macd1 - trigger1;
			serieHistogram.add(date1, histogram);

		}

	}

	/**
	 * Modifizierter XYBarRenderer. Positive Werte werden gr&uuml;n dargestellt,
	 * negative rot.
	 */
	static class MyXYBarRenderer extends XYBarRenderer {

		/**
		 * serial UID
		 */
		private static final long serialVersionUID = -8978350984656959134L;

		/**
		 * Standardkonstruktor, ruft nur super() auf
		 */
		public MyXYBarRenderer() {
			super();
		}

		/**
		 * @see org.jfree.chart.renderer.xy.XYItemRenderer#getItemPaint(int row,
		 *      int column)
		 */
		@Override
		public Paint getItemPaint(final int series, final int item) {
			Paint color;
			// here we assume we're working with the primary dataset
			final XYDataset dataset = getPlot().getDataset(2);
			final double value = dataset.getYValue(series, item);
			if (value >= 0) {
				color = Color.green;
			} else {
				color = Color.red;
			}
			return color;
		}
	}

}
